<?php

declare(strict_types=1);

namespace Dedi\SyliusBlogPlugin\Repository;

use Doctrine\ORM\QueryBuilder;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;
use Sylius\Component\Core\Model\ChannelInterface;

abstract class AbstractRepository extends EntityRepository
{
    protected function createTranslationBasedQueryBuilder(?string $locale = null): QueryBuilder
    {
        $queryBuilder = $this->createQueryBuilder('o')
            ->addSelect('translation')
            ->innerJoin('o.translations', 'translation', 'WITH', 'translation.locale = :locale')
        ;

        if (null !== $locale) {
            $queryBuilder
                ->andWhere('translation.locale = :locale')
                ->setParameter('locale', $locale)
            ;
        }

        return $queryBuilder;
    }

    protected function createBaseQueryBuilder(ChannelInterface $channel, ?string $locale = null): QueryBuilder
    {
        return $this->createTranslationBasedQueryBuilder($locale)
            ->andWhere(':channel MEMBER OF o.channels')
            ->andWhere('o.enabled = true')
            ->orderBy('o.position', 'ASC')
            ->setParameter('channel', $channel)
        ;
    }

    /**
     * @return mixed
     *
     * @throws \Doctrine\ORM\NonUniqueResultException
     */
    public function findOneByChannelAndSlug(ChannelInterface $channel, string $slug, ?string $locale = null)
    {
        return $this->createBaseQueryBuilder($channel, $locale)
            ->andWhere('translation.slug = :slug')
            ->setParameter('slug', $slug)
            ->getQuery()
            ->getOneOrNullResult()
        ;
    }
}
